/*
 * ***************************************************************************
 * File:           netacuity_api.c
 * Author:         Digital Envoy
 * Program Name:   netacuity_api
 * Version:        3.0
 * Date:           02/25/02
 *
 *  Description:
 *    These functions allow a developer to access the NetAcuity Databases.
 *    For function docs see the na_api.h file
 *
 *
 *  Revision History:
 *  ----------------------------------
 *  $Log: netacuity_api.c,v $
 *  Revision 1.2  2004/01/17 01:39:33  bhuffake
 *
 *  	I need to add extern strtok_r to netacuity_api.c.
 *
 *  Revision 1.1  2004/01/17 00:26:38  bhuffake
 *
 *      Added code that will allows sk_analysis_dump to talk to netacuity_api.
 *
 *  Revision 1.1  2003/10/16 19:15:47  bhuffake
 *
 *  	Adding netacq to the cvs repository.
 *
 *  Revision 1.5  2002/07/02 13:54:45  jnystrom
 *  Made additions for new Databases.
 *
 *  Revision 1.4  2002/05/30 22:14:30  jnystrom
 *  The information from the server is made to be 1496 so the total data will be 1500.
 *
 *  Revision 1.3  2002/05/30 21:35:07  jnystrom
 *  Allowed for a max response from the server to be 1500 bytes.
 *
 *  Revision 1.2  2002/05/28 09:20:25  tmcneese
 *  Make sure to check the array boundaries when using MAX_FEATURES.
 *
 *  Revision 1.1  2002/05/14 14:51:26  tmcneese
 *  Move APIs to individual directories.
 *
 *  Revision 3.4  2002/03/26 09:26:25  tmcneese
 *  Added strtok to api directory. Solaris and Freebsd need this.
 *
 *  Revision 3.3  2002/03/23 00:08:09  jnystrom
 *  Made default response to NA if db_feature > 500 or less than 0.
 *
 *  Revision 3.2  2002/03/15 23:52:11  jnystrom
 *  Changed the return codes on the new functions.
 *
 *  Revision 3.1  2002/03/01 23:03:25  jnystrom
 *  Made function for domain and sic db querying and got rid of "personum" references
 *
 *  This is the first changes for NetAcuity 3.0.  This allows the new query
 *  to the the current NA db's (geo, sic, domain) and keeps backward
 *  compatibility.
 *-------------------------------------------------------------------------
 * 
* **************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <sys/time.h> 
#include <sys/uio.h> 
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <strings.h>

#include "netacuity_api.h"

extern "C" {
    extern char *strtok_r(char *s1, const char *s2, char **lasts);
}

#define API_SERVER_UDP_PORT 5400
#define API_VERSION 0
#define MAX_RESPONSE_SIZE 1496
// global variables
static struct in_addr api_server; // Initialized by api_set_server_addr()
static int api_timeout = 1000000; // 1000ms
static int api_id = 0; // unregistered

/* Struct to recv response into */
typedef struct api_response_t
{
    short size;                        // size of the response including NULL
    short num_response_fields;         // number of fields in response
    char response[MAX_RESPONSE_SIZE];  // response
}api_response;

/************** private routines *********************/

// Function:    set_nonblock_flag
// Returns:     result of calling fcntl
// Parameters:  file descripter, value (0-off/1-on)
// Description: Changes the state of the fd's non_block flag

static int set_nonblock_flag (int desc, int value)
{
    int oldflags = fcntl (desc, F_GETFL, 0);
    /* If reading the flags failed, return error indication now. */
    if (oldflags == -1)
        return -1;
    /* Set just the flag we want to set. */
    if (value != 0)
        oldflags |= O_NONBLOCK;
    else
        oldflags &= ~O_NONBLOCK;
    /* Store modified flag word in the descriptor. */
    return fcntl (desc, F_SETFL, oldflags);
}

/********** API Routines ************************/
void na_api_set_id(int id)
{
    api_id = id;  // truncate the string if it didn't fit
}

// returns 0 if invalid address
int na_api_set_server_addr(char *addr)
{
    // inet_addr returns -1 on invalid address
    api_server.s_addr = inet_addr(addr);
    return (api_server.s_addr > 0 ? 1 : 0);
}

void na_api_set_timeout(int t)
{
    api_timeout = t;
}

// returns 1 if okay, 0 if error
int na_query_raw(char *ip, char *response, int *num_fields, int *size,
                 int db_feature)
{
    char request_buff[256];  // this will be the string we send to the server
    char resp_buff[256];     // this is the server's response to parse
    int sockfd; // socket
    int ret;    // hold various function return values
    fd_set fds; // descriptor set for select()
    struct timeval tv; // time structure
    struct sockaddr_in dest_addr;   /* will hold the destination addr */
    int l;  // length parameter for send() call
    api_response resp;
    unsigned int dest_len;
    
    
   
    /* zero buffers */
    bzero(request_buff, sizeof(request_buff));
    bzero(resp_buff, sizeof(resp_buff));

    *size = 0;
    *num_fields = 0;
    strcpy(response, "NA\n");
    /* Make sure the db to be queried is not greater than 500 or less than 1.*/
    if ((db_feature >= 500) || (db_feature < 1))
    {
        return 0;
    }
    /* create and check socket, uses UDP */
    /* TCP is not supported any more */
    ret = sockfd = socket(AF_INET, SOCK_DGRAM, 0);      
    dest_addr.sin_port = htons(API_SERVER_UDP_PORT);
    if(ret == -1) return 0;

    /* make it non-blocking, so we can timeout without using signals */
    ret = set_nonblock_flag(sockfd, 1);
    if(ret == -1) return 0;
  
    /* initialize IP connect structure */
    dest_addr.sin_family = AF_INET;        /* host byte order */
    /* PORT number was setup above */
    dest_addr.sin_addr = api_server;
    bzero(&(dest_addr.sin_zero), 8);       /* zero the rest of the struct */
  
    /* make timeout structure */
    tv.tv_sec = 0;
    tv.tv_usec = api_timeout;
  
    /* Deal with Solaris issue with US being > 1000000 (1 sec) */
    while (tv.tv_usec >= 1000000)
    {
        tv.tv_usec -= 1000000;
        tv.tv_sec++;
    }

    // we're connected okay
 
    // request format is: db_feature;api_id;ip_addr;\015\012

    // make request string
    sprintf(request_buff, "%d;%d;%s;\015\012",
            db_feature,
            api_id,
            ip);

    dest_len = sizeof(struct sockaddr);
    // send string to socket
    l = strlen(request_buff);
    ret=sendto(sockfd, request_buff, l, 0, (struct sockaddr *)&dest_addr,
               dest_len);
    if(ret < l)
    {
        /* this shouldn't happen, bail out */
        close(sockfd);
        return 0;
    }

    // wait for the response data
    FD_ZERO(&fds);
    FD_SET(sockfd, &fds);
  
    tv.tv_sec = 0;
    tv.tv_usec = api_timeout;
  
    /* Deal with Solaris issue with US being > 1000000 (1 sec) */
    while (tv.tv_usec >= 1000000)
    {
        tv.tv_usec -= 1000000;
        tv.tv_sec++;
    }

    /* wait until the timeout or we have data from the server */
    ret=select(sockfd+1, &fds, NULL, NULL, &tv);

    /* For error or timeout */
    if(ret < 1)
    {
        close(sockfd);
        return 0;
    }
    /* Receive information from server */
    ret = recvfrom(sockfd, &resp, sizeof(resp), 0,
                   (struct sockaddr *)&dest_addr,
                   &dest_len);
    /* Check to see if got actual good data off socket */
    if (ret < 6)
    {
        return 0;
    }
    /* Fill in the pointers passed in */
    *size = ntohs((int)(resp.size));
    
    *num_fields = ntohs((int)(resp.num_response_fields)); 
    
    strcpy(response, resp.response);
    // printf("The response: %s", response);
    
    
    // close the socket
    close(sockfd);
    return 1;  // success         
}


int parse_geo_string(char *geo_string, na_geo_struct *r)
{
    char *field;             // this is a temp variable for parsing
    char *temp_buf;
    // try to parse the result, the response format is:
    // usa;pa;pittsburgh;?;95;85;80;0;80.001;-45.445;

    if(!(field=(char *)strtok_r(geo_string, ";", &temp_buf)))
    {
        return 0;
    }
    strncpy(r->country, field, 3);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    strncpy(r->region, field, 31);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    strncpy(r->city, field, 31);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    strncpy(r->speed, field, 10);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    r->country_c=atoi(field);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    r->region_c=atoi(field);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    r->city_c=atoi(field);
    
    /* for Backward compatibility just let the
     * metro code default to 0 if not running 
     * this version 
     */
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf))) return 1;
    r->metro_code=atoi(field);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf))) return 1;
    r->latitude=atof(field);
    if(!(field=(char *)strtok_r(NULL, ";", &temp_buf))) return 1;
    r->longitude=atof(field);
    
    return 1;
}

/* Query the NA Geo Db */
int na_query_geo(char *ip, na_geo_struct *r)
{
    char response[256];
    int num_fields;
    int size;

    if(!r) return 0;  // check parameter

     /* Initialize response structure */
    bzero(r, sizeof(na_geo_struct));
    strcpy(r->country, "?");
    strcpy(r->region, "?");
    strcpy(r->city, "?");
    strcpy(r->speed, "?");
    
    if (!na_query_raw(ip, response, &num_fields, &size, NA_GEO))
    {
        return 0;
    }
    if (size == 0)
    {
        return 0;
    }
  
    if (!parse_geo_string(response, r))
    {
        return 0;
    }
    return 1;
}

/* Query the NA Domain Db */
int na_query_domain(char *ip, na_domain_struct *r)
{
    char response[256];
    int size;
    int num_fields;
    char *temp_buf;
    char *field;
    
    if (r == NULL)
    {
        return 0;
    }
    bzero(r, sizeof(na_domain_struct));
    strcpy(r->domain, "?");

    if (!na_query_raw(ip, response, &num_fields, &size, NA_DOMAIN))
    {
        return 0;
    }

    if (size == 0)
    {
        return 0;

    }
    if (!(field = (char *)strtok_r(response, ";", &temp_buf)))
    {
        return 0;
    }
    strncpy(r->domain, field, 255);
    return 1;
}

/* Query the NA SIC Db */
int na_query_sic(char *ip, na_sic_struct *r)
{
    char response[256];
    int size;
    int num_fields;
    char *temp_buf;
    char *field;
    
    if (r == NULL)
    {
        return 0;
    }
    /* Default SIC is 0 */
    bzero(r, sizeof(na_sic_struct));
    r->sic = 0;

    if (!na_query_raw(ip, response, &num_fields, &size, NA_SIC))
    {
        return 0;
    }

    if (size == 0)
    {
        return 0;

    }
    if (!(field = (char *)strtok_r(response, ";", &temp_buf)))
    {
        return 0;
    }
    r->sic = atoi(field);
    return 1;

}
/* Query the NA Zip DB
 * fills in the area code, zip, and timezone info
 */
int na_query_zip(char *ip, na_zip_struct *r)
{
    char response[256];
    int size;
    int num_fields;
    char *temp_buf;
    char *field;

    if (r == NULL)
    {
        return 0;
    }
    /* Default zip response is 0;0;9999;?; */
    bzero(r, sizeof(na_zip_struct));
    r->zip = 0;
    r->area_code = 0;
    r->current_offset = 9999;
    strcpy(r->in_dst,"?");
    
    if (!na_query_raw(ip, response, &num_fields, &size, NA_ZIP))
    {
        return 0;
    }

    if (size == 0)
    {
        return 0;
    }
    /* Parse out the area_code */
    if (!(field = (char *)strtok_r(response, ";", &temp_buf)))
    {
        return 0;
    }
    r->area_code = atoi(field);
    /* Parse out zip */
    if (!(field = (char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    r->zip = atoi(field);
    /* Parse out offset from UTC(GMT) */
    if (!(field = (char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    r->current_offset = atoi(field);
    /* Parse out in daylight savings time */
    if (!(field = (char *)strtok_r(NULL, ";", &temp_buf)))
    {
        return 0;
    }
    strcpy(r->in_dst, field);
    return 1;
}

/* Query the NA ISP Db */
int na_query_isp(char *ip, na_isp_struct *r)
{
    char response[256];
    int size;
    int num_fields;
    char *temp_buf;
    char *field;

    if (r == NULL)
    {
        return 0;
    }
    /* Default isp response is ?; */
    bzero(r, sizeof(na_isp_struct));
    strcpy(r->isp,"?");
    
    if (!na_query_raw(ip, response, &num_fields, &size, NA_ISP))
    {
        return 0;
    }

    if (size == 0)
    {
        return 0;
    }
    /* Parse out ISP from response */
    if (!(field = (char *)strtok_r(response, ";", &temp_buf)))
    {
        return 0;
    }
    strcpy(r->isp, field);
    return 1;
}

/* Query NA Home/Business Db */
int na_query_home_biz(char *ip, na_home_biz_struct *r)
{
    char response[256];
    int size;
    int num_fields;
    char *temp_buf;
    char *field;

    if (r == NULL)
    {
        return 0;
    }
    /* Default isp response is ?; */
    bzero(r, sizeof(na_home_biz_struct));
    strcpy(r->home_biz,"?");
    
    if (!na_query_raw(ip, response, &num_fields, &size, NA_HOME_BIZ))
    {
        return 0;
    }

    if (size == 0)
    {
        return 0;
    }
    /* Parse out home/business info */
    if (!(field = (char *)strtok_r(response, ";", &temp_buf)))
    {
        return 0;
    }
    strcpy(r->home_biz, field);
    return 1;
}
